import os
import shutil
import tempfile
from unittest import TestCase


class BaseTestCase(TestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self._temp_files = []
        self._temp_dirs = []

    def tearDown(self):
        super(BaseTestCase, self).tearDown()
        self.clean_temp_files()

    def make_temp_file(self):
        ignore, file = tempfile.mkstemp()
        self._temp_files.append(file)
        return self._temp_files[-1]

    def make_temp_dir(self):
        self._temp_dirs.append(tempfile.mkdtemp())
        return self._temp_dirs[-1]

    def clean_temp_files(self):
        for file in self._temp_files:
            if os.path.exists(file):
                os.unlink(file)
        for dir in self._temp_dirs:
            if os.path.exists(dir):
                shutil.rmtree(dir)
